import os

import decorator

def catching(exception_type=Exception, handler=None):
    def catching(f, *args, **kw):
        try:
            return f(*args, **kw)
        except exception_type as e:
            if handler:
                handler(e)

    return decorator.decorator(catching)

def fullpath(path):
    return os.path.abspath(
        os.path.expanduser(path))



import ConfigParser
import datetime, logging, os

from .feed import Feed
from .util import fullpath

logger = logging.getLogger('yoink.config')

class Config(object):
    def __init__(self, config_file, preview=False):
        self.config_file = fullpath(config_file)
        self.cp = ConfigParser.ConfigParser()

        logger.info('loading config file: {0}'.format(config_file))
        self.cp.read(self.config_file)

        self.preview = preview

    def close(self):
        with open(self.config_file, 'w') as f:
            logger.info('writing config file: {0}'.format(self.config_file))
            self.cp.write(f)

    def get(self, section, option):
        return self.cp.get(section, option)

    def set(self, section, option, value):
        self.cp.set(section, option, value)

    def has_section(self, section):
        return self.cp.has_section(section)

    def feeds(self):
        for section in self.cp.sections():
            if section.startswith('feed:'):
                yield FeedProxy(self, section)

    def __enter__(self):
        return self

    def __exit__(self, t, v, tb):
        if t is None:
            self.close()

class FeedProxy(object):
    def __init__(self,
                 config,
                 section):
        assert config.has_section(section)

        self.config = config
        self.section = section

    def __get_timestamp(self):
        try:
            timestamp = self.config.get(self.section, 'timestamp')
            timestamp = datetime.datetime.strptime(
                self.config.get(self.section, 'timestamp'),
                Feed.TIME_FORMAT)

            if timestamp.year < 1900:
                timestamp = datetime.datetime(1900, 1, 1)

        except ConfigParser.NoOptionError:
            timestamp = datetime.datetime(1900, 1, 1)
            
        return timestamp

    def __set_timestamp(self, timestamp):
        self.config.set(
            self.section,
            'timestamp',
            timestamp.strftime(Feed.TIME_FORMAT))

    url = property(lambda self: self.config.get(self.section, 'url'),
                   lambda self, val: self.config.set(self.section, 'url', val))
    folder = property(lambda self: self.config.get(self.section, 'folder'),
                      lambda self, val: self.config.set(self.section, 'folder', val))
    timestamp = property(__get_timestamp,
                         __set_timestamp)

    def update(self):
        feed = Feed(
            url=self.url,
            folder=fullpath(os.path.join(self.config.get('config', 'folder'), self.folder)),
            timestamp=self.timestamp,
            config=self.config)

        feed.update()

        self.timestamp = feed.timestamp
        



import ConfigParser, datetime, logging, os, time, urllib, urlparse

import feedparser

from .util import catching

logger = logging.getLogger('yoink.feed')

class Feed(object):
    TIME_FORMAT = '%Y-%m-%dT%H:%M:%S'

    def __init__(self,
                 url,
                 folder,
                 timestamp,
                 config):
        self.url = url
        self.folder = folder
        self.timestamp = timestamp
        self.config = config

    @catching(handler=lambda e: logger.error('Error processing feed: {1}'.format(e)))
    def update(self):
        def entry_sort(x, y):
            if x['updated_parsed'] < y['updated_parsed']:
                return -1
            elif x['updated_parsed'] > y['updated_parsed']:
                return 1
            else:
                return 0

        logger.info('reading feed: {0}'.format(self.url))
              
        # read the feed
        parsed = feedparser.parse(self.url)

        # grab the entries
        entries = parsed['entries']

        # filter out entries older than the feed timestamp
        filter(lambda e: e['updated_parsed'] > self.timestamp.timetuple(), 
               entries)

        # sort entries in ascending timestamp order
        entries.sort(entry_sort)

        # process each entry (in timestamp order)
        map(self.process_entry, parsed['entries'])

    @catching(exception_type=KeyError, handler=lambda e: logger.error('Entry is missing a field: {0}'.format(e)))
    def process_entry(self, entry):
        # Download the entry if it's timestamp is greater than the
        # feed's
        if entry['updated_parsed'][:8] > self.timestamp.timetuple()[:8]:
            for enc in entry['enclosures']:
                if enc['type'].startswith('audio/'):

                    if self.config.preview:
                        logger.info('[PREVIEW] Downloading {0}'.format(enc['href']))

                    else:
                        self.download(enc['href'])
                    
                        # update the feed's timestamp with the entry's
                        self.timestamp = datetime.datetime.strptime(
                            time.strftime(
                                Feed.TIME_FORMAT, 
                                entry['updated_parsed']),
                            Feed.TIME_FORMAT)

    def download(self, url):
        try:
            os.makedirs(self.folder)
        except OSError:
            pass

        filename = os.path.split(
            urlparse.urlparse(url).path)[1]

        dest = os.path.join(self.folder, filename)

        logger.info('Downloading {0} to {1}'.format(url, dest))
        try:
            urllib.urlretrieve(url, dest)

        except Exception as e:
            logger.error('Error downloading entry {0}: {1}'.format(url, e))



from ConfigParser import ConfigParser
import logging, sys

import baker

from .config import Config

logger = logging.getLogger('yoink.app')

logging.basicConfig(level=logging.DEBUG,
                    stream=sys.stdout,
                    format='%(message)s')

@baker.command(default=True)
def update(config_file='~/.yoink.conf', preview=False):
    with Config(config_file, preview) as config:
        for feed in config.feeds():
            feed.update()

def main():
    baker.run()
